@@ -68,6 +68,8 @@ pub enum FinalError {
     Args(#[from] ArgsError),
     #[error(transparent)]
     Interaction(#[from] InteractivityError),
+    #[error(transparent)]
+    Request(#[from] RequestError), // should only ever be fatal
     #[error("no search results :(")]
     NoSearchResults,
     #[error("failed to format output as requested: {0}")]
@@ -83,11 +85,11 @@ impl FinalError {
         2 for program error
          */
         match self {
+            NoSearchResults => 0,
             Args(_) => 1,
+            Request(_) | FormatOutput(_) => 2,
             // 0 if non-fatal, 2 if fatal
             Interaction(inner) => (inner.is_fatal() as i32) * 2,
-            NoSearchResults => 0,
-            FormatOutput(_) => 2,
         }
     }
 }
@@ -179,8 +181,12 @@ pub struct MediaTypeParseError(pub String);
 pub enum InteractivityError {
     #[error("user aborted operation")]
     Cancel,
-    #[error("unexpected CLI error {0}\nIf you were just trying to stop running the program, please create an issue about this")]
+    #[error("unexpected CLI error: {0}\nIf you were just trying to stop running the program, please create an issue about this")]
     Dialoguer(io::Error),
+    #[error("unexpected crossterm error: {0}")]
+    Crossterm(io::Error),
+    #[error("unexpected TUI error: {0}")]
+    Tui(io::Error),
 }
 
 impl MaybeFatal for InteractivityError {
@@ -189,8 +195,8 @@ impl MaybeFatal for InteractivityError {
     }
 }
 
-impl From<io::Error> for InteractivityError {
-    fn from(err: io::Error) -> Self {
+impl InteractivityError {
+    pub fn from_cli(err: io::Error) -> Self {
         use InteractivityError::*;
         match err.kind() {
             io::ErrorKind::NotConnected => Cancel,
@@ -209,6 +215,13 @@ pub enum RequestError {
     Omdb(String),
 }
 
+impl MaybeFatal for RequestError {
+    fn is_fatal(&self) -> bool {
+        use RequestError::Omdb;
+        !matches!(self, Omdb(_))
+    }
+}
+
 #[derive(Debug, Error)]
 pub enum SignUpError {
     #[error(transparent)]
